// check rhino
try {
    if ( window ) {
        _print = alert;
    } else {
        _print = print;
    }
} catch ( e ) {
    _print = print;
}

function str( key ) {
    try {
        var value = localizedStrings[ key ];
        if ( value ) return value;
        debug( "no value" );
        return key;
    } catch ( e ) {
        debug( "no localizedStrings" );
        return key;
    }
}

function __inherit_class( myClass, superClass ) {
    //_print( "inheritting " + superClass );
    myClass.prototype.superClass = superClass;
    for ( var key in superClass.prototype ) {
        if ( myClass.prototype[ key ] == null ) {
            //_print( "inheritting " + key );
            myClass.prototype[ key ] = superClass.prototype[ key ];
            //_print( myClass.prototype[ key ] );
        }
    }
}

function Product() {
    this.asin      = null;
    this.name      = null;
    this.detailedURL = null;
    this.category  = null;
    this.icon      = { url:null, width:null, height:null };
    this.price     = { amount:0, currency:"USD", formatted:"" };
    this.priceLowestNew  = { amount:0, currency:"USD", formatted:"" };
    this.priceLowestUsed = { amount:0, currency:"USD", formatted:"" };
    this.salesRank = null;
    this.manufacturer = null;
    this.availability = null;
}

Product.prototype.countryCode = "US";

Product.prototype.getMaximumTitleLength = function() {
    if ( this.countryCode == "JP" ) {
        return 48;
    }
    return 64;
}

Product.prototype.getMaximumDescriptionLength = function() {
    if ( this.countryCode == "JP" ) {
        return 16;
    }
    return 32;
}

//Product.prototype.setName = function( name ) { this.name = name; };
Product.prototype.setSalesRank = function( salesrank ) { 
    rank = parseInt( salesrank );
    if ( rank ) {
        this.salesRank = rank;
    } else {
        //debug( "no rank " + salesrank + " => " + rank );
        this.salesRank = null;
    }
};
Product.prototype.setAvailability = function( avail ) {
    this.availability = avail;
};
Product.prototype.getAvailability = function() {
    return this.availability;
};
Product.prototype.setCategory = function( category ) { this.category = category; };
Product.prototype.setASIN = function( asin ) { this.asin = asin; };
Product.prototype.setTitle = function( title ) { this.title = title; };
Product.prototype.setPrice = function( amount, currency, formatted ) {
    this.price.amount = parseInt( amount );
    this.price.currency = currency;
    if ( formatted ) {
        this.price.formatted = formatted;
    } else if ( this.price.currency != null && this.price.amount != null ) {
        this.price.formatted = amount + "" + currency;
    } else {
        this.price.formatted = ""
    }
};
Product.prototype.setLowestNewPrice = function( amount, currency, formatted ) {
    //debug( "lowest new price " + formatted );
    this.priceLowestNew.amount = parseInt( amount );
    this.priceLowestNew.currency = currency;
    if ( formatted ) {
        this.priceLowestNew.formatted = formatted;
    } else if ( this.priceLowestNew.currency != null && this.priceLowestNew.amount != null ) {
        this.priceLowestNew.formatted = amount + "" + currency;
    } else {
        this.priceLowestNew.formatted = ""
    }
}
Product.prototype.setLowestUsedPrice = function( amount, currency, formatted ) {
    //debug( "lowest used price " + formatted );
    this.priceLowestUsed.amount = parseInt( amount );
    this.priceLowestUsed.currency = currency;
    if ( formatted ) {
        this.priceLowestUsed.formatted = formatted;
    } else if ( this.priceLowestUsed.currency != null && this.priceLowestUsed.amount != null ) {
        this.priceLowestUsed.formatted = amount + "" + currency;
    } else {
        this.priceLowestUsed.formatted = "-"
    }
}

Product.prototype.setIcon = function( url, width, height ) {
    this.icon.url = url;
    this.icon.width = parseInt( width );
    this.icon.height = parseInt( height );
    if ( this.icon.width == null || this.icon.height == null || this.icon.url == null ) {
        this.icon = { url:null, width:null, height:null };
    }
};
Product.prototype.setURL = function( url ) { 
    this.detailedURL = url; 
};
Product.prototype.setManufacturer = function( manu ) { 
    this.manufacturer = manu;
};


//Product.prototype.getName    = function() { return this.name; };
Product.prototype.getSalesRank = function() { return this.salesRank; };
Product.prototype.getCategory = function() { return this.category; };
Product.prototype.getASIN = function() { return this.asin; };
Product.prototype.getTitle   = function() { return this.title; };
Product.prototype.getURL     = function() { return this.detailedURL; };
Product.prototype.getIconURL = function() { this.icon.url; };
Product.prototype.hasIcon    = function() { return ( this.icon.url != null ); };
Product.prototype.getIconSize= function() { return this.icon; };
Product.prototype.getManufacturer = function() { return this.manufacturer; };
Product.prototype.hasPrice = function() {
    if ( this.priceLowestNew.amount > 0 ) {
        return true;
    } else if ( this.price.amount > 0 ) {
        return true;
    } else if ( this.priceLowestUsed.amount > 0 ) {
        return true;
    }
    return false;
}

Product.prototype.createPriceElement = function() {
    var aprice = this.getAmazonPriceElement();
    var mprice = this.getMarketPriceElement();
    var div = document.createElement( "div" );
    div.setAttribute( "class", "prices" );
    if ( aprice ) {
        div.appendChild( aprice );
    } 
    if ( mprice ) {
        div.appendChild( mprice );
    }
    return div;
};

Product.prototype.getAmazonPriceElement = function() {
    if ( this.price.amount > 0 ) {
        var div = document.createElement( "div" );
        div.setAttribute( "class", "price-amazon" );
        div.innerText = this.price.formatted;
        //debug( this.title + " : amazon price " + this.price.formatted );
        return div;
    } else {
        return null;
    }
};

Product.prototype.getMarketPriceElement = function() {
    var div = document.createElement( "div" );
    div.setAttribute( "class", "price-market" );
    if ( this.priceLowestNew.amount > 0 ) {
        var pnew = document.createElement( "span" );
        pnew.setAttribute( "class", "lowestnewprice" );
        pnew.innerText = this.priceLowestNew.formatted;
        div.appendChild( pnew );
        if ( this.priceLowestUsed.amount > 0 ) {
            div.appendChild( document.createTextNode( " / " ) );
        }
    }
    if ( this.priceLowestUsed.amount > 0 ) {
        var pused = document.createElement( "span" );
        pused.setAttribute( "class", "lowestusedprice" );
        pused.innerText = this.priceLowestUsed.formatted;
        div.appendChild( pused );
    }
    return div;
};

Product.prototype.getPrice   = function() { 
    if ( this.price.amount > 0 ) {
        return this.price.formatted;
    } else if ( this.priceLowestNew.amount > 0 ) {
        return this.priceLowestNew.formatted;
    } else if ( this.priceLowestUsed.amount > 0 ) {
        return this.priceLowestUsed.formatted;
    }
    return null;
};

Product.prototype.createCategoryElement = function() {
    if ( this.category ) {
        var elem = document.createElement( "div" );
        elem.setAttribute( "class", "category" );
        elem.appendChild( document.createTextNode( this.category ) );
        if ( this.salesRank ) {
            var rank = document.createElement( "span" );
            var numeral = null;
            var remnant = this.salesRank % 100;
            if ( remnant == 11 ) { 
                numeral = localizedStrings[ "NUMERAL_11" ]; 
            } else if ( remnant == 12 ) { 
                numeral = localizedStrings[ "NUMERAL_12" ]; 
            } else {
                remnant = remnant % 10;
                if ( remnant == 1 ) { 
                    numeral = localizedStrings[ "NUMERAL_1" ]; 
                } else if ( remnant == 2 ) { 
                    numeral = localizedStrings[ "NUMERAL_2" ]; 
                } else if ( remnant == 3 ) { 
                    numeral = localizedStrings[ "NUMERAL_3" ]; 
                } else {
                    numeral = localizedStrings[ "NUMERAL_GENERAL" ]; 
                }
            }
            if ( numeral == null ) {
                numeral = "";
            }
            rank.setAttribute( "class", "salesRank" );
            rank.appendChild( document.createTextNode( this.salesRank + numeral ) );
            elem.appendChild( rank );
        }
        return elem;
    } else {
        return null;
    }
}

Product.prototype.createIconElement = function( iconSize ) {
    var elem = document.createElement( "div" );
    elem.setAttribute( "class", "thumbnail" );
    var img = document.createElement( "img" );
    elem.appendChild( img );
    if ( this.hasIcon() ) {
        //debug( "ICON : " + this.icon.url );
        var width = iconSize.width;
        var height = iconSize.height;
        var ratio = width / height;
        var tratio = this.icon.width / this.icon.height;
        img.src = this.icon.url;
        if ( ratio > tratio ) {
            img.height = height;
            img.width = parseInt( height * tratio );
            var left = ( width - img.width ) / 2;
            img.setAttribute( "style", "margin-left:" + left + "px" );//positon:absolute;left:" + left + "px;top:0px;" );
        } else {
            img.width = width;
            img.height = parseInt( width / tratio );
            var top = ( height - img.height ) / 2;
            img.setAttribute( "style", "margin-top:" + top + "px" );
            //img.setAttribute( "style", "positon:absolute;left:0px;top:" + top + "px;" );
        }
    } else {
        img.src = "url(Images/NoImage.png)";
        //img.style.background = "gray";
        img.width = iconSize.width;
        img.height = iconSize.height;
    }
    return elem;
};

Product.prototype.getAssociateURL = function() {
    if ( this.detailedURL.indexOf( "wobsofweewidg" ) >= 0 ) {
        return this.detailedURL;
    }
    //debug( "current country code " + Product.prototype.countryCode + " " + this.countryCode );
    if ( Product.prototype.countryCode == "JP" ) {
        return "http://www.amazon.co.jp/exec/obidos/redirect?path=ASIN/" + this.asin + "&tag=wobsofweewidg-22&link_code=as2&camp=1789&creative=9325";
    } else if ( Product.prototype.countryCode == "UK" ) {
        return "http://www.amazon.co.uk/exec/obidos/redirect?path=ASIN/" + this.asin + "&tag=wobsofweewidg-21&link_code=as2&camp=1789&creative=9325";
    } else {
        return "http://www.amazon.com/exec/obidos/redirect?path=ASIN/" + this.asin + "&tag=wobsofweewidg-20&link_code=as2&camp=1789&creative=9325";
    }
};

Product.prototype.createTitleElement = function() {
    var link;
    link = document.createElement( "span" );
    link.setAttribute( "class", "title" );
    if ( this.title == null ) this.title = "Title";
    var title = this.title;
    if ( title.length > this.getMaximumTitleLength() ) {
        title = title.substring( 0, this.getMaximumTitleLength() - 2 ) + "..";
    }
    link.appendChild( document.createTextNode( title ) );
    return link;
};

Product.prototype.createDescriptionElement = function() {
    var desc = document.createElement( "div" );
    var link = this.createTitleElement();
    desc.appendChild( link );
    //desc.appendChild( document.createElement( "br" ) );
    desc.appendChild( this.createPriceElement() );
    if ( this.manufacturer ) {
        var manu = document.createElement( "div" );
        manu.setAttribute( "class", "manufacturer" );
        desc.appendChild( manu );
        manu.appendChild( document.createTextNode( this.manufacturer ) );
        //desc.appendChild( document.createElement( "br" ) );
    }
    return desc;
};

Product.prototype.createHTMLElement = function( descriptionSize, iconSize ) {
    if ( iconSize == null ) {
        iconSize = { width:40, height:50 };
    }
    var elem = document.createElement( "div" );
    if ( descriptionSize ) {
        elem.setAttribute( "style", "width:" + descriptionSize.width + ";height:" + descriptionSize.height );
//         elem.style.width = descriptionSize.width;
//         elem.style.height = descriptionSize.height;
    }
    var icon = this.createIconElement( iconSize );
    icon.setAttribute( "class", "thumbnail" );
    elem.appendChild( icon );
    var desc = this.createDescriptionElement();
    desc.setAttribute( "class", "description" );
    if ( this.availability ) {
        var avl = document.createElement( "div" );
        avl.setAttribute( "class", "availability" );
        avl.innerText = this.availability;
        desc.appendChild( avl );
    }
    elem.appendChild( desc );
    var cat = this.createCategoryElement();
    if ( cat ) {
        elem.appendChild( cat );
    }
    return elem;
};

Product.prototype.importAttributes = function( attr ) {
    this.manufacturer = getInsideTexts( attr, "", "Manufacturer" );
    return;
};

function BookProduct() {
    this.asin      = null;
    this.name      = null;
    this.detailedURL = null;
    this.icon      = { url:null, width:null, height:null };
    this.price     = { amount:0, currency:"USD", formatted:"-" };
    this.priceLowestNew  = { amount:0, currency:"USD", formatted:"-" };
    this.priceLowestUsed = { amount:0, currency:"USD", formatted:"-" };
    this.authors = new Array();
    this.publisher = null;
    this.salesRank = 999999;
}

BookProduct.prototype.appendAuthor = function( author ) {
    this.authors.push( author );
};

BookProduct.prototype.setPublisher = function( publisher ) {
    this.publisher = publisher;
};

BookProduct.prototype.getAuthors = function() {
    return this.authors;
};

BookProduct.prototype.getPublisher = function() { 
    return this.publisher; 
};

BookProduct.prototype.getAuthor = function() { 
    return this.authors[ 0 ]; 
};

BookProduct.prototype.importAttributes = function( attr ) {
    this.setPublisher( getInsideTexts( attr, '', "Publisher" ) );
    //debug( "publisher " + this.publisher );
    var num = 1;
    while ( true ) {
        var author = getInsideTexts( attr, '', "Author," + num );
        if ( author == null ) break;
        this.authors.push( author );
        //debug( "author " + num + "," + author );
        num ++;
        //       break;
    }
    //debug( "author " + this.getAuthor() + ", pub " + this.getPublisher() );
    //outputDOMText( attr, debug );
    return;
};

BookProduct.prototype.createDescriptionElement = function() {
    var desc = document.createElement( "div" );
    var link = this.createTitleElement();//document.createElement( "a" );

    //link.setAttribute( "href", this.getAssociateURL() );
    desc.appendChild( link );
    //desc.appendChild( document.createElement( "br" ) );
    desc.appendChild( this.createPriceElement() );
//     if ( this.hasPrice() ) {
//         desc.appendChild( document.createTextNode( this.getPrice() ) );
//         desc.appendChild( document.createElement( "br" ) );
//     }
    if ( this.getPublisher() ) {
        var publisher = document.createElement( "div" );
        publisher.setAttribute( "class", "publisher" );
        publisher.appendChild( document.createTextNode( this.getPublisher()) );
        desc.appendChild( publisher );//document.createElement( "br" ) );
    }
    if ( this.authors != null && this.authors.length > 0 ) {
        var author = document.createElement( "div" );
        var line = "";
        author.setAttribute( "class", "author" );
        line += this.authors[ 0 ];
        //author.appendChild( document.createTextNode( this.authors[ 0 ] ) );
        for ( var i = 1; i < this.authors.length; i++ ) {
            line += ", " + this.authors[ i ];
            //author.appendChild( document.createTextNode( ", " + this.authors[ i ] ) );
        }
        if ( line.length > this.getMaximumDescriptionLength() ) {
            line = line.substring( 0, this.getMaximumDescriptionLength() - 2 ) + "..";
        }
        author.appendChild( document.createTextNode( line ) );
        desc.appendChild( author );
    }
    return desc;
};

__inherit_class( BookProduct, Product );

function DVDProduct() {
    this.asin      = null;
    this.name      = null;
    this.detailedURL = null;
    this.icon      = { url:null, width:null, height:null };
    this.price     = { amount:0, currency:"USD", formatted:"-" };
    this.priceLowestNew  = { amount:0, currency:"USD", formatted:"-" };
    this.priceLowestUsed = { amount:0, currency:"USD", formatted:"-" };
    this.salesRank = 999999;
    this.studio = null;
    this.runningTime = null;
    this.UPC = null;
    this.regionCode = null;
    this.format = null;
}

DVDProduct.prototype.importAttributes = function( attr ) {
    this.studio = getInsideTexts( attr, '', "Studio" );
    var relem = getElementAt( attr, "RunningTime" );
    if ( relem ) {
        var unit = relem.getAttribute( "units" );
        this.runningTime = relem.innerText;
        if ( unit == "hours" ) {
            this.runningTime *= 60;
        }
    }
    this.UPC = getInsideTexts( attr, '', "UPC" );
    this.regionCode = getInsideTexts( attr, '', "RegionCode" );
    this.format = getInsideTexts( attr, "", "Format" );
};

DVDProduct.prototype.createDescriptionElement = function() {
    var desc = document.createElement( "div" );
    var link = this.createTitleElement();
    desc.appendChild( link );
    desc.appendChild( this.createPriceElement() );
    if ( this.studio ) {
        var studio = document.createElement( "div" );
        studio.setAttribute( "class", "studio" );
        studio.appendChild( document.createTextNode( this.studio ) );
        desc.appendChild( studio );
    }
    if ( this.runningTime ) {
        var time = document.createElement( "div" );
        time.setAttribute( "class", "time" );
        time.appendChild( document.createTextNode( "(" + this.runningTime + "min)" ) );
        desc.appendChild( time );
    }
    return desc;
};

__inherit_class( DVDProduct, Product );

function VideoGameProduct() {
    this.asin      = null;
    this.name      = null;
    this.detailedURL = null;
    this.icon      = { url:null, width:null, height:null };
    this.price     = { amount:0, currency:"USD", formatted:"$0" };
    this.priceLowestNew  = { amount:0, currency:"USD", formatted:"-" };
    this.priceLowestUsed = { amount:0, currency:"USD", formatted:"-" };
    this.platform  = new Array();
    this.publisher = null;
}

VideoGameProduct.prototype.importAttributes = function( attr ) {
    //this.publisher = getInsideTexts( attr, "", "Publisher" );
    var num = 1;
    while ( true ) {
        var elem = getElementAt( attr, "Platform," + num );
        if ( elem ) {
            this.platform.push( getInnerText( elem ) );
            //this.platform.push( elem.innerText );//getInsideText( elem ) );
            num ++;
        } else {
            break;
        }
    }
};

VideoGameProduct.prototype.createDescriptionElement = function() {
    var desc = document.createElement( "div" );
    var link = this.createTitleElement();
    desc.appendChild( link );
    //desc.appendChild( document.createElement( "br" ) );
    desc.appendChild( this.createPriceElement() );
    if ( this.platform.length > 0 ) {
        var platform = document.createElement( "div" );
        platform.setAttribute( "class", "platform" );
        var line = "for " + this.platform[ 0 ];
        for ( var i = 1; i < this.platform.length; i++ ) {
            line += "," + this.platform[ i ];
        }
        if ( line.length > this.getMaximumDescriptionLength() ) {
            line = line.substring( 0, this.getMaximumDescriptionLength() - 2 ) + "..";
        }
        platform.appendChild( document.createTextNode( line ) );
        //desc.appendChild( document.createElement( "br" ) );
        desc.appendChild( platform );
    }
    if ( this.manufacturer ) {
        desc.appendChild( this.createManufacturerElement() );
    }
    return desc;
};

Product.prototype.createManufacturerElement = function() {
    if ( this.manufacturer ) {
        var manufacturer = document.createElement( "div" );
        manufacturer.setAttribute( "class", "manufacturer" );
        manufacturer.appendChild( document.createTextNode( this.manufacturer ) );
        return manufacturer;
    }
    return null;
};

__inherit_class( VideoGameProduct, Product );

function BrandProduct() {
    this.asin      = null;
    this.name      = null;
    this.detailedURL = null;
    this.icon      = { url:null, width:null, height:null };
    this.price     = { amount:0, currency:"USD", formatted:"$0" };
    this.priceLowestNew  = { amount:0, currency:"USD", formatted:"-" };
    this.priceLowestUsed = { amount:0, currency:"USD", formatted:"-" };
    this.brand = null;
}

BrandProduct.prototype.importAttributes = function( attr ) {
    this.brand = getInsideTexts( attr, "", "Brand" );
};

BrandProduct.prototype.createDescriptionElement = function() {
    var desc = document.createElement( "div" );
    if ( this.brand ) {
        var brand = document.createElement( "div" );
        brand.setAttribute( "class", "brand" );
        brand.appendChild( document.createTextNode( this.brand ) );
        desc.appendChild( brand );//document.createElement( "br" ) );
    }
    var link = this.createTitleElement();
    desc.appendChild( link );
    //desc.appendChild( document.createElement( "br" ) );
    desc.appendChild( this.createPriceElement() );
    return desc;
};

__inherit_class( BrandProduct, Product );

function ToyProduct() {
    this.asin      = null;
    this.name      = null;
    this.detailedURL = null;
    this.icon      = { url:null, width:null, height:null };
    this.price     = { amount:0, currency:"USD", formatted:"$0" };
    this.priceLowestNew  = { amount:0, currency:"USD", formatted:"-" };
    this.priceLowestUsed = { amount:0, currency:"USD", formatted:"-" };
    this.manufacturer = null;
    this.minimumAge  = null;
    this.maximumAge = null;
    this.ageUnits   = "";
}

ToyProduct.prototype.importAttributes = function( attr ) {
    this.manufacturer = getInsideTexts( attr, "", "Manufacturer" );
    var maelem = getElementAt( attr, "ManufacturerMinimumAge" );
    if ( maelem ) {
        var units = maelem.getAttribute( "units" );
        if ( units == null ) units = "months";
        this.ageUnits = units;
        this.minimumAge = parseInt( getInnerText( maelem ) );
        maelem = getElementAt( attr, "ManufacturerMaximumAge" );
        if ( maelem ) {
            this.maximumAge = parseInt( getInnerText( maelem ) );
        }
    }
    var aaelem = getElementAt( attr, "AmazonMinimumAge" );
    if ( aaelem ) {
        var units = aaelem.getAttribute( "units" );
        if ( units == null ) units = "months";
        this.ageUnits = units;
        this.minimumAge = parseInt( getInnerText( aaelem ) );
        aaelem = getElementAt( attr, "AmazonMaximumAge" );
        if ( aaelem ) {
            this.maximumAge = parseInt( getInnerText( aaelem.innerText ) );
        }
    }
    //debug( "TOY " + this.manufacturer + ", " + this.minimumAge + ", " + this.maximumAge );
};

ToyProduct.prototype.createDescriptionElement = function() {
    var desc = document.createElement( "div" );
    var link = this.createTitleElement();
    desc.appendChild( link );
    //desc.appendChild( document.createElement( "br" ) );
    desc.appendChild( this.createPriceElement() );
    if ( this.manufacturer ) {
        desc.appendChild( this.createManufacturerElement() );
    }
    if ( this.minimumAge ) {
        var age = document.createElement( "div" );
        age.setAttribute( "class", "age" );
        if ( this.maximumAge == null || isNaN( this.maximumAge ) ) {
            age.appendChild( document.createTextNode( str( "Minimum Age" ) + ": " + this.minimumAge + " " + str( this.ageUnits ) ) );
            desc.appendChild( age );
            //desc.appendChild( document.createElement( "br" ) );
        } else {
            age.appendChild( document.createTextNode( str( "Age" ) + ": " + this.minimumAge + " - " + this.maximumAge + " " + str( this.ageUnits ) ) );
            desc.appendChild( age );
        }
    }
    return desc;
};

__inherit_class( ToyProduct, Product );


function MusicProduct() {
    this.asin      = null;
    this.name      = null;
    this.detailedURL = null;
    this.icon      = { url:null, width:null, height:null };
    this.price     = { amount:0, currency:"USD", formatted:"$0" };
    this.priceLowestNew  = { amount:0, currency:"USD", formatted:"-" };
    this.priceLowestUsed = { amount:0, currency:"USD", formatted:"-" };
    this.artists = null;
    this.creators = null;
    this.label = null;
}

MusicProduct.prototype.importAttributes = function( attr ) {
    this.label = getInsideTexts( attr, "", "Label" );

    {
        var elem = getElementAt( attr, "Artist" );
        if ( elem ) {
            var num = 1;
            this.artists = new Array();
            do {
                var name = getInnerText( elem );
                this.artists.push( name );
                num ++;
                elem = getElementAt( attr, "Artist," + num );
            } while ( elem );
        }
    }
            

    {
        var elem = getElementAt( attr, "Creator" );
        if ( elem ) {
            var num = 1;
            this.creators = new Array();
            while ( true ) {
                elem = getElementAt( attr, "Creator," + num );
                if ( elem == null ) break;
                var name = getInnerText( elem );
                name.role = elem.attributes.item( 0 ).value;
                //debug( name + " / " + name.role );
                this.creators.push( name );
                num ++;
            }
        }
    }
};

MusicProduct.prototype.createDescriptionElement = function() {
    var desc = document.createElement( "div" );
    var link = this.createTitleElement();
//     if ( this.brand ) {
//         desc.appendChild( document.createTextNode( this.brand ) );
//         desc.appendChild( document.createElement( "br" ) );
//     }
    //link.appendChild( document.createTextNode( this.title ) );
    desc.appendChild( link );
    //desc.appendChild( document.createElement( "br" ) );
    desc.appendChild( this.createPriceElement() );
    // music
    if ( this.artists != null && this.artists.length > 0 ) {
        var artists = document.createElement( "div" );
        artists.setAttribute( "class", "artist" );
        var line = this.artists[ 0 ];
        for ( var i = 1; i < this.artists.length; i++ ) {
            line += ", " + this.artists[ i ];
        }
        if ( line.length > this.getMaximumDescriptionLength() ) {
            line = line.substring( 0, this.getMaximumDescriptionLength() ) + "..";
        }
        artists.appendChild( document.createTextNode( line ) );
        desc.appendChild( artists );
    } else if ( this.creators != null && this.creators.length > 0 ) {
        var creator = document.createElement( "div" );
        creator.setAttribute( "class", "creator" );
        var line = this.creators[ 0 ];
        for ( var i = 1; i < this.creators.length; i++ ) {
            line += ", " + this.creators[ i ];
        }
        if ( line.length > this.getMaximumDescriptionLength() ) {
            line = line.substring( 0, this.getMaximumDescriptionLength() ) + "..";
        }
        creator.appendChild( document.createTextNode( line ) );
        desc.appendChild( creator );
    }
    if ( this.label ) {
        var label = document.createElement( "div" );
        label.setAttribute( "class", "label" );
        label.appendChild( document.createTextNode( this.label ) );
        desc.appendChild( label );
    }
    return desc;
};

__inherit_class( MusicProduct, Product );
